@@ -68,7 +68,7 @@ class Example(object):
 
     # Depth of this example in the example tree. The top-level example has a
     # depth of 0.
-    depth = attr.ib()
+    depth = attr.ib(repr=False)
 
     # A label is an opaque value that associates each example with its
     # approximate origin, such as a particular strategy class or a particular
@@ -84,16 +84,16 @@ class Example(object):
     # An example is "trivial" if it only contains forced bytes and zero bytes.
     # All examples start out as trivial, and then get marked non-trivial when
     # we see a byte that is neither forced nor zero.
-    trivial = attr.ib(default=True)
+    trivial = attr.ib(default=True, repr=False)
 
     # True if we believe that the shrinker should be able to delete this
     # example completely, without affecting the value produced by its enclosing
     # strategy. Typically set when a rejection sampler decides to reject a
     # generated value and try again.
-    discarded = attr.ib(default=None)
+    discarded = attr.ib(default=None, repr=False)
 
     # List of child examples, represented as indices into the example list.
-    children = attr.ib(default=attr.Factory(list))
+    children = attr.ib(default=attr.Factory(list), repr=False)
 
     @property
     def length(self):
@@ -119,11 +119,11 @@ class Block(object):
     # True if this block's byte values were forced by a write operation.
     # As long as the bytes before this block remain the same, modifying this
     # block's bytes will have no effect.
-    forced = attr.ib()
+    forced = attr.ib(repr=False)
 
     # True if this block's byte values are all 0. Reading this flag can be
     # more convenient than explicitly checking a slice for non-zero bytes.
-    all_zero = attr.ib()
+    all_zero = attr.ib(repr=False)
 
     @property
     def bounds(self):
@@ -138,6 +138,15 @@ class Block(object):
         return self.forced or self.all_zero
 
 
+class _Overrun(object):
+    status = Status.OVERRUN
+
+    def __repr__(self):
+        return "Overrun"
+
+
+Overrun = _Overrun()
+
 global_test_counter = 0
 
 
@@ -161,6 +170,7 @@ class ConjectureData(object):
         self.block_starts = {}
         self.blocks = []
         self.buffer = bytearray()
+        self.index = 0
         self.output = u""
         self.status = Status.VALID
         self.frozen = False
@@ -182,6 +192,13 @@ class ConjectureData(object):
         top = self.start_example(TOP_LABEL)
         assert top.depth == 0
 
+    def __repr__(self):
+        return "ConjectureData(%s, %d bytes%s)" % (
+            self.status.name,
+            len(self.buffer),
+            ", frozen" if self.frozen else "",
+        )
+
     def __assert_not_frozen(self, name):
         if self.frozen:
             raise Frozen("Cannot call %s on frozen ConjectureData" % (name,))
@@ -192,10 +209,6 @@ class ConjectureData(object):
         # that as depth 0 rather than depth 1.
         return len(self.example_stack) - 1
 
-    @property
-    def index(self):
-        return len(self.buffer)
-
     def all_block_bounds(self):
         return [block.bounds for block in self.blocks]
 
@@ -291,6 +304,7 @@ class ConjectureData(object):
             assert isinstance(self.buffer, hbytes)
             return
         self.finish_time = benchmark_time()
+        assert len(self.buffer) == self.index
 
         while self.example_stack:
             self.stop_example()
@@ -336,9 +350,9 @@ class ConjectureData(object):
         return result
 
     def write(self, string):
+        string = hbytes(string)
         self.__assert_not_frozen("write")
         self.__check_capacity(len(string))
-        assert isinstance(string, hbytes)
         original = self.index
         self.__write(string, forced=True)
         self.forced_indices.update(hrange(original, self.index))
@@ -371,6 +385,7 @@ class ConjectureData(object):
         assert len(result) == n
         assert self.index == initial
         self.buffer.extend(result)
+        self.index = len(self.buffer)
         self.stop_example()
 
     def draw_bytes(self, n):
